//
//  UserDefaults.swift
//  SampleApp
//
//  Created by Federico Zanetello on 10/29/18.
//  Copyright © 2018 David Zhang. All rights reserved.
//

import Foundation
import AmitySDK

extension UserDefaults {
    
    typealias ChannelId = String

    var filter: AmityChannelQueryFilter {
        get { return AmityChannelQueryFilter(rawValue: filterInt) ?? .all }
        set { filterInt = newValue.rawValue }
    }
    
    var channelTypeFilter: AmityChannelType {
        get { return AmityChannelType(rawValue: channelTypeFilterInt) ?? .standard }
        set { channelTypeFilterInt = newValue.rawValue }
    }
    
    private var channelTypeFilterInt: Int {
        get { return Int(integer(forKey: #function)) }
        set { set(newValue, forKey: #function) }
    }
    
    private var filterInt: Int {
        // If the value is absent or can't be converted to an integer, 0 will be returned.
        get { return integer(forKey: #function) }
        set { set(newValue, forKey: #function)}
    }

    var excludingTags: [String] {
        get { return stringArray(forKey: #function) ?? [] }
        set { set(newValue, forKey: #function) }
    }

    var includingTags: [String] {
        get { return stringArray(forKey: #function) ?? [] }
        set { set(newValue, forKey: #function) }
    }

    var userId: String? {
        get { return string(forKey: #function) }
        set { set(newValue, forKey: #function) }
    }

    var deviceToken: String? {
        get { return string(forKey: #function) }
        set { set(newValue, forKey: #function) }
    }
    
    var currentEnvironment: ApiEnvironment? {
        return getSavedEnvironments().first(where: { $0.id == currentEnvironmentId })
    }
    
    var currentEnvironmentId: String? {
        get { return string(forKey: #function) }
        set { set(newValue, forKey: #function) }
    }

    var parentId: String? {
        get { return string(forKey: #function) }
        set { set(newValue, forKey: #function) }
    }

    var channelReversePreference: [ChannelId: Bool] {
        get { return dictionary(forKey: #function) as? [ChannelId: Bool] ?? [:] }
        set { set(newValue, forKey: #function) }
    }

    var channelPreferenceIncludingTags: [ChannelId: [String]] {
        get { return dictionary(forKey: #function) as? [ChannelId: [String]] ?? [:] }
        set { set(newValue, forKey: #function) }
    }

    var channelPreferenceExcludingTags: [ChannelId: [String]] {
        get { return dictionary(forKey: #function) as? [ChannelId: [String]] ?? [:] }
        set { set(newValue, forKey: #function) }
    }

    var channelPreferenceFilterByParentIdActive: [ChannelId: Bool] {
        get { return dictionary(forKey: #function) as? [ChannelId: Bool] ?? [:] }
        set { set(newValue, forKey: #function) }
    }

    var channelPreferenceFilterByParentId: [ChannelId: String] {
        get { return dictionary(forKey: #function) as? [ChannelId: String] ?? [:] }
        set { set(newValue, forKey: #function) }
    }
    
    var channelPreferenceFilterByType: [ChannelId: Int?] {
        get { return dictionary(forKey: #function) as? [ChannelId: Int?] ?? [:] }
        set { set(newValue, forKey: #function) }
    }
    
    // If true, can call registerDevice
    // If false, cannot call registerDevice
    //
    // It should be false only when user call unregister device manually
    // When user logout it can be true or false depend on the current value
    var isRegisterdForPushNotification: Bool {
        get { return bool(forKey: #function) }
        set { set(newValue, forKey: #function) }
    }
    
    func getSavedEnvironments() -> [ApiEnvironment] {
        var savedEnvironments = [ApiEnvironment]()
        if let data = self.value(forKey: "saved_environments") as? Data, let savedData = try? PropertyListDecoder().decode([ApiEnvironment].self, from: data) {
            savedEnvironments = savedData
        }
        return savedEnvironments
    }
    
    func saveEnvironments(_ environments: [ApiEnvironment]) {
        if let data = try? PropertyListEncoder().encode(environments) {
            self.set(data, forKey: "saved_environments")
        }
    }
    
    func addEnvironment(_ environment: ApiEnvironment) {
        var savedEnvironments = getSavedEnvironments()
        savedEnvironments.append(environment)
        saveEnvironments(savedEnvironments)
    }
    
    func updateEnvironment(_ environment: ApiEnvironment) {
        var savedEnvironments = getSavedEnvironments()
        if let index = savedEnvironments.firstIndex(where: { $0.id == environment.id }) {
            savedEnvironments[index] = environment
        }
        saveEnvironments(savedEnvironments)
    }
    
}
