//
//  ChannelRealTimeEventManager.swift
//  SampleApp
//
//  Created by Amity on 11/10/2565 BE.
//  Copyright © 2565 BE David Zhang. All rights reserved.
//

import Foundation
import AmitySDK

class ChannelRealTimeEventManager {
    
    enum Element {
        case header
    }
    
    var channelToken: AmityNotificationToken?
    var channelRepo = AmityChannelRepository(client: AmityManager.shared.client!)
    
    var channel: AmityChannel
    var elements = [ChannelRealTimeEventManager.Element]()
    var topicSubscriber: AmityTopicSubscription
    
    var observeChannelChanges: (() -> Void)?
    
    init(channel: AmityChannel) {
        self.channel = channel
        self.elements = [.header]
        self.topicSubscriber = AmityTopicSubscription(client: AmityManager.shared.client!)
    }
    
    func subscribeEvent(completion: ((Bool) -> Void)?) {
        observeCurrentChannel()
        
        let channelTopic = AmityChannelTopic(channel: channel)
        
        print("Channel Topic Path: \(channelTopic.getSubscriptionTopic(AmityManager.shared.client!))")
        
        self.topicSubscriber.subscribeTopic(channelTopic) { success, error in
            completion?(success)
            Log.add(info: "Channel Subscribe Event: \(success) Error: \(String(describing: error))")
        }
    }
    
    func unsubscribeEvent(completion: ((Bool) -> Void)?) {
        let channelTopic = AmityChannelTopic(channel: channel)
        
        self.topicSubscriber.unsubscribeTopic(channelTopic) { success, error in
            completion?(success)
            Log.add(info: "Channel UnSubscribe Event: \(success) Error: \(String(describing: error))")
        }
    }
    
    func observeCurrentChannel() {
        channelToken?.invalidate()
        channelToken = channelRepo.getChannel(channel.channelId).observe({ [weak self] liveChannel, error in
            guard let channel = liveChannel.object else { return }
            
            self?.channel = channel
            self?.observeChannelChanges?()
        })
    }
    
    deinit {
        channelToken?.invalidate()
    }
}
