//
//  SingleCommunityView.swift
//  SampleApp
//
//  Created by Nishan Niraula on 6/23/22.
//  Copyright © 2022 David Zhang. All rights reserved.
//

import SwiftUI
import AmitySDK

struct SingleCommunityView: View {
    
    @State var commId: String = ""
    @State var commDetails: String = ""
    @State var token: AmityNotificationToken?
    
    var commRepo = AmityCommunityRepository(client: AmityManager.shared.client!)
    
    var body: some View {
        
        VStack {
            TextField("Enter Community Id", text: $commId)
                .frame(height: 50)
                .textFieldStyle(RoundedBorderTextFieldStyle())
            
            Button {
                fetchCommunity()
            } label: {
                Text("Fetch")
                    .padding()
                    .frame(width: 150, height: 45)
                    .foregroundColor(Color.white)
                    .background(Color.blue)
                    .clipped()
                    .cornerRadius(8)
            }
            .padding(.top, 24)

            Text(commDetails)
            
            Spacer()
        }
        .padding()
    }
    
    func fetchCommunity() {
        guard !commId.isEmpty else {
            commDetails = "Please enter community id first."
            return
        }
        
        token?.invalidate()
        commDetails = ""
        
        token = commRepo.getCommunity(withId: commId).observe({ liveObject, error in
            
            guard liveObject.dataStatus == .fresh, let community = liveObject.object else { return }
            
            self.commDetails = community.modelDescription
        })
    }
}

struct SingleCommunityView_Previews: PreviewProvider {
    static var previews: some View {
        SingleCommunityView()
    }
}
