//
//  CreateCommentView.swift
//  SampleApp
//
//  Created by Nishan Niraula on 13/2/2566 BE.
//  Copyright © 2566 BE David Zhang. All rights reserved.
//

import SwiftUI
import PhotosUI
import Combine

struct CreateCommentView: View {
    
    /*
     - Mentions
     - Images
     - Text
     
     */
    //let createAction: (() -> Void)?
    
    @State private var commentText: String = ""
    @State private var parentId: String = "" // Parent Id
    @State private var showPhotoPicker: Bool = false
    @State private var selectedImages: [UIImage] = []
    @State private var capturedImage: UIImage?
    @State private var showCamera: Bool = false
    
    var columns = [GridItem(.adaptive(minimum: 80), spacing: 8, alignment: .center)]
    
    var body: some View {
        ScrollView {
            VStack(alignment: .leading) {
                Text("Comment")
                    .font(.headline)
                
                TextField("Comment...", text: $commentText)
                    .padding(.horizontal)
                    .font(.system(size: 15, weight: .regular))
                    .frame(height: 50)
                    .foregroundColor(Color(uiColor: .systemGroupedBackground))
                    .background(Color(uiColor: .systemGroupedBackground))
                    .cornerRadius(8)
                
                HStack {
                    Text("Add Images")
                        .font(.headline)
                    
                    Spacer()
                    
                    Button {
                        showPhotoPicker = true
                    } label: {
                        Image(systemName: "photo")
                    }
                    .padding(8)
                    .background(Color(uiColor: UIColor.systemGroupedBackground))
                    .cornerRadius(8)
                    
                    Button {
                        showCamera = true
                    } label: {
                        Image(systemName: "camera.fill")
                    }
                    .padding(8)
                    .background(Color(uiColor: UIColor.systemGroupedBackground))
                    .cornerRadius(8)
                    .padding(.leading, 8)
                }
                .padding(.top, 16)
                
                LazyVGrid(columns: columns) {
                    ForEach(selectedImages, id: \.self) { item in
                        Image(uiImage: item)
                            .resizable()
                            .scaledToFit()
                            .frame(width: 80, height: 80)
                    }
                }
                
                HStack {
                    Spacer()
                    
                    Button {
                        print("Image Count: \(selectedImages.count)")
                    } label: {
                        Text("Create Comment")
                    }
                    .padding(8)
                    .buttonStyle(.bordered)
                    
                    Spacer()
                }
                .padding(.top, 32)
                
            }
            .padding(16)
        }
        .sheet(isPresented: $showPhotoPicker) {
            PhotoPicker(pickerResult: $selectedImages, isPresented: $showCamera)
        }
        .sheet(isPresented: $showCamera) {
            CameraPicker(image: $capturedImage)
        }
        .onChange(of: capturedImage) { newValue in
            guard let newValue else { return }
            
            selectedImages.append(newValue)
        }
    }
}

struct CreateCommentView_Previews: PreviewProvider {
    static var previews: some View {
        CreateCommentView()
    }
}
