//
//  TestController.swift
//  SampleApp
//
//  Created by Nishan Niraula on 10/15/20.
//  Copyright © 2020 David Zhang. All rights reserved.
//

import Foundation
import UIKit
import AmitySDK
import Combine

// NOTE:
// Ignore stuffs in this class. This class is just for quick testing of sdk stuffs inside sample app.
class TestController: UIViewController {
    
    let client = AmityManager.shared.client!
    
    var token: AmityNotificationToken?
    
    let userRepo = AmityUserRepository(client: AmityManager.shared.client!)
    let commRepo = AmityCommunityRepository(client: AmityManager.shared.client!)
    let commentRepo = AmityCommentRepository(client: AmityManager.shared.client!)
    let postRepo = AmityPostRepository(client: AmityManager.shared.client!)
    let channelRepo = AmityChannelRepository(client: AmityManager.shared.client!)
    let messageRepo = AmityMessageRepository(client: AmityManager.shared.client!)
    let fileRepo = AmityFileRepository(client: AmityManager.shared.client!)
    let subChannelRepo = AmitySubChannelRepository(client: AmityManager.shared.client!)
    let reactionRepo = AmityReactionRepository(client: AmityManager.shared.client!)
    
    var messageEditor: AmityMessageEditor?
    var messageFlagger: AmityMessageFlagger?
    
    let userPresenceRepo = AmityUserPresenceRepository(client: AmityManager.shared.client!)
    let channelPresenceRepo = AmityChannelPresenceRepository(client: AmityManager.shared.client!)
    
    var disposeBag: Set<AnyCancellable> = []
    
    override func loadView() {
        view = UIView()
        view.backgroundColor = .white
    }
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        AmityLog.logLevel = .all
        
        let button1 = UIBarButtonItem(title: "A1", style: .plain, target: self, action: #selector(onButton1Tap))
        let button2 = UIBarButtonItem(title: "A2", style: .plain, target: self, action: #selector(onButton2Tap))
        let button3 = UIBarButtonItem(title: "A3", style: .plain, target: self, action: #selector(onButton3Tap))
        let button4 = UIBarButtonItem(title: "A4", style: .plain, target: self, action: #selector(onButton4Tap))
        
        self.navigationItem.rightBarButtonItems = [button1, button2, button3, button4]
    }
    
    var channels = [AmityChannel]()

    @objc func onButton1Tap() {
        let channelQuery = AmityChannelQuery()
        channelQuery.types = [AmityChannelQueryType.conversation]
        channelQuery.filter = .userIsMember
        
        token = channelRepo.getChannels(with: channelQuery).observe({ [weak self] liveCollection, change, error in
            Log.add(info: "Channel Fetched: \(liveCollection.count()) | Data Status: \(liveCollection.dataStatus.description)")
            self?.channels = liveCollection.allObjects()
        })
    }
    
    @objc func onButton2Tap() {
        for channel in channels {
            channelPresenceRepo.syncChannelPresence(id: channel.channelId)
        }
        
        channelPresenceRepo.getSyncingChannelPresence().sink { completion in
            
            
        } receiveValue: { presences in
            
            Log.add(info: "-- Presence Observer Triggered --")

            for presence in presences {
                Log.add(info: "Channel Id: \(presence.channelId)")
                Log.add(info: "User Presences: \(presence.userPresences.map { $0.userId} )")
            }
        }
        .store(in: &disposeBag)
    }
    
    @objc func onButton3Tap() {
        Log.add(info: "Snapshot Users Count: \(snapshot?.users.count)")
        Log.add(info: "Snapshot Fetched At: \(snapshot?.fetchedAt)")
        //channelPresenceRepo.unsyncChannelPresence(id: channels[0].channelId )
    }
    
    var snapshot: OnlineUsersSnapshot?

    @objc func onButton4Tap() {
        Task {
            do {
                snapshot = try await userPresenceRepo.getOnlineUsersSnapshot()
                Log.add(info: "Snapshot Users: \(snapshot?.users.count)")
            } catch let error {
                Log.add(info: "Errorrrr occurred")
            }
        }
    }
}
